'===================================
'= Asynchronous Socket Client Code =
'===================================
'
'Manages the communication from the source TCP client and the CrystalFontz display
'
'Written by Stuart Massey (VorTechS)
'3rd February 2010

'Some portions of this code have been taken from MSDN examples
'http://msdn.microsoft.com/en-us/library/bew39x2a(VS.71).aspx


'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Imports System
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Threading
Imports Microsoft.VisualBasic
Imports PhatHack_LCD_Driver.AsynchronousSocketState

Public Class AsynchronousSocketClient

    Inherits Socket

    'Localised instance of a DriverClient
    Dim _sourceClient As DriverClient = Nothing

    Public Sub New(ByVal SourceClient As DriverClient)

        'Creates a new TCP socket
        MyBase.new(Sockets.AddressFamily.InterNetwork, Sockets.SocketType.Stream, Sockets.ProtocolType.Tcp)

        'Bind the source client to this instance
        _sourceClient = SourceClient

    End Sub

    Public Sub ReadCallback(ByVal ar As IAsyncResult)

        Dim content As String = String.Empty

        ' Retrieve the state object and the handler socket
        ' from the asynchronous state object.
        Dim state As AsynchronousSocketState = CType(ar.AsyncState, AsynchronousSocketState)
        Dim handler As Socket = state.workSocket

        If handler.Connected Then
            Try
                ' Read data from the client socket. 
                Dim bytesRead As Integer = handler.EndReceive(ar)

                If bytesRead > 0 Then
                    ' There  might be more data, so store the data received so far.
                    state.sb.Append(Encoding.ASCII.GetString(state.buffer, 0, bytesRead))

                    ' Put the socket back in the receive state, ready for when more data is sent
                    handler.BeginReceive(state.buffer, 0, AsynchronousSocketState.BufferSize, 0, New AsyncCallback(AddressOf ReadCallback), state)
                End If

                'If we have some data in the buffer....
                If state.sb.Length > 0 Then
                    '....go ahead and parse it
                    Call ParseCommands(state)
                End If
            Catch ex As Exception
                'Avoid any exceptions where the socket disappears unexpectedly...
            End Try
        End If

    End Sub

    Public Overloads Sub Send(ByVal handler As Socket, ByVal data As String)

        'Our client requires that commands are CRLF terminated.  
        'Because you can't rely on the client correctly terminating the command, we'll
        'ensure it is at the point of sending
        If Not (data.EndsWith(vbCrLf)) Then
            data &= vbCrLf
        End If

        ' Convert the string data to byte data using ASCII encoding.
        Dim byteData As Byte() = Encoding.ASCII.GetBytes(data)

        ' Begin sending the data to the remote client
        handler.BeginSend(byteData, 0, byteData.Length, 0, New AsyncCallback(AddressOf SendCallback), handler)

    End Sub

    Private Sub SendCallback(ByVal ar As IAsyncResult)

        ' Retrieve the socket from the state object.
        Dim handler As Socket = CType(ar.AsyncState, Socket)

        ' Complete sending the data to the remote client.
        Dim bytesSent As Integer = handler.EndSend(ar)

    End Sub

    Private Sub SendClientHelp(ByVal sourceclient As DriverClient)

        'A simple function to send help text to remote clients. 
        'This allows you to test the service using a telnet, or other socket connecting client and get a little help
        'on the command syntax.
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "PhatHack CrystalFontz 634 LCD Driver")
        Call CommonFunctions.SendClientText(sourceclient, "Version: " & My.Application.Info.Version.ToString() & " - HELP")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "Syntax: COMMAND [/PARAM] [VALUE]")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "Available Commands:")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "START /port <x> = Start the display on the specified port.")
        Call CommonFunctions.SendClientText(sourceclient, "CLEAR = Clear LCD Display.")
        Call CommonFunctions.SendClientText(sourceclient, "CLEARLINE /line <x> = Clears the content buffer for line [x]")
        Call CommonFunctions.SendClientText(sourceclient, "CONTENT /line <x> = Clears the content buffer for line [x]")
        Call CommonFunctions.SendClientText(sourceclient, "DISCONNECT = Disconnect the connected LCD Display.")
        Call CommonFunctions.SendClientText(sourceclient, "FLASH /line <x> = Update the flash inidicator for line [x]")
        Call CommonFunctions.SendClientText(sourceclient, "SCROLL /line <x> = Update the scroll positions for line [x]")
        Call CommonFunctions.SendClientText(sourceclient, "UPDATE = Updates the display with all contents of the content buffer")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "QUIT = Disconnect from this service.")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "")
        Call CommonFunctions.SendClientText(sourceclient, "Waiting for commands (type HELP if using a terminal)")

    End Sub

    Private Sub ParseCommands(ByVal state As AsynchronousSocketState)

        'Take a copy of whatever we have in the buffer
        Dim CurrentBuffer As String = state.sb.ToString()

        'Find out where the last validate break point is 
        '(the point at which data is validly terminated)
        Dim validLength As Integer = ValidateBuffer(CurrentBuffer)

        'Get the valid part of the buffer.. and adjust the current client buffer accordingly
        If validLength > -1 Then

            'Get the valid portion of the buffer only
            Dim validBuffer As String = CurrentBuffer.Substring(0, validLength)

            'CRLF = 2 characters, and the end point is the beginning of these character
            'so we need to account for this fact when adjusting the buffer
            state.sb.Remove(0, validLength + 2)

            'Split the current buffer to get the commands
            Dim commandSet() As String = validBuffer.Split(vbCrLf)

            'Iterate each one
            For Each command As String In commandSet

                'Split, so we can also evaluate the command parameters
                Dim params() As String = command.Split("/")

                If params.Length > 0 Then
                    Select Case params(0).ToUpper.Trim
                        Case "HELP" : Call SendClientHelp(_sourceClient)
                        Case "QUIT" : Call CommonFunctions.ShutDownClient(_sourceClient)
                        Case "START" : Call StartDisplay(_sourceClient, params)
                        Case "CLEAR" : Call ClearDisplay(_sourceClient)
                        Case "CLEARLINE" : Call ClearLine(_sourceClient, params)
                        Case "CONTENT" : Call UpdateStyleContent(_sourceClient, params)
                        Case "DISCONNECT" : Call DisconnectDisplay(_sourceClient)
                        Case "RESET" : Call ResetContent(_sourceClient)
                        Case "SCROLL" : Call ModifyScrollIndex(_sourceClient, params)
                        Case "FLASH" : Call ModifyFlashIndicator(_sourceClient, params)
                        Case "UPDATE" : Call Update(_sourceClient)
                    End Select
                End If

            Next command
        End If

    End Sub

    'Disconnects the selected display, and destroys the instance if no other clients are using it
    Public Sub DisconnectDisplay(ByVal sourceClient As DriverClient)

        'Get the display instance
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)
        Dim isMultiInstanced As Boolean = False

        'It's possible to disconnect before it's even initialized... this check gets round it
        'but will probably leave you with a connected instance of a display which may have
        'incorrect settings....
        If Not (sourceDisplay Is Nothing) Then

            'Check all other connected client
            For Each otherClient As DriverClient In CommonFunctions.ConnectedClients

                If otherClient.DisplayPort = sourceDisplay.Port And Not (otherClient Is sourceClient) Then
                    isMultiInstanced = True
                    Exit For
                End If
            Next otherClient

            'If we have to, close down the display
            If Not (isMultiInstanced) Then
                Call sourceDisplay.Close()

                'Remove it from the collection of displays
                Call CommonFunctions.ConnectedDisplays.Remove(sourceDisplay)
            End If

            'Now, shut down this socket
            Call CommonFunctions.ShutDownClient(sourceClient)
        End If

        'Give any socket that just started the connection a chance to catchup
        System.Threading.Thread.Sleep(1000)

    End Sub

    'Reset the display, clearing all styles etc
    Private Sub ResetContent(ByVal SourceClient As DriverClient)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then
            Call sourceDisplay.Reset()
        End If

    End Sub

    'Force the display to be updated, by first updating the LCD buffer contents
    'according to the current 'style' and then sending the resulting bytes
    'out to the display
    Private Sub Update(ByVal SourceClient As DriverClient)

        'One bug that appears to be present, is that there is a slight timing issue
        'somewhere - that means the state of the MultiStyles collection seems to get
        'modified on a reset, whilst this update code can be being fired.

        'It's not harmful, so we can simply ignore such errors
        Try
            'Get the current display for the source client
            Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

            If Not (sourceDisplay Is Nothing) Then
                'Run through each multi-object in order and do an update
                For Each LCDLine As MultiStyleLine In sourceDisplay.MultiStyles

                    'Force the buffer to be updated to what it should be right now
                    Call LCDLine.DisplayStyle()

                    'Now request the buffer contents to send to the display
                    Dim sourceLineBytes() As Byte = LCDLine.GetBytes()

                    'As long as there is something to display
                    If Not (sourceLineBytes Is Nothing) Then
                        'Send out the buffer to the display
                        Call sourceDisplay.DirectWrite(sourceLineBytes)
                    End If

                    'Reset the buffer to ensure there is no staleness
                    LCDLine.SetBytes(Nothing)
                Next LCDLine
            End If
        Catch
        End Try

    End Sub

    'Validates the socket buffer content to return an index at which there is 
    'validly terminated content
    Private Function ValidateBuffer(ByVal CurrentBuffer As String) As Integer

        'Run through the buffer and determine how much of it is valid, so we can
        'leave anything invalid untouched in the hope whatever sent it - catches up
        Return CurrentBuffer.LastIndexOf(vbCrLf)

    End Function

    'Start the specified display if it isn't already
    Private Sub StartDisplay(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        Dim ComPort As Integer = -1
        Dim BaudRate As Integer = -1
        Dim Parity As Integer = -1
        Dim StopBits As Integer = -1
        Dim DataBits As Integer = -1
        Dim DTROn As Integer = 0
        Dim RTSOn As Integer = 0

        For Each param As String In DisplayParams

            Dim paramValues() As String = param.Split(Chr(32))

            Dim paramName As String = paramValues(0)
            Dim ParamValue As String = paramValues(1)

            Select Case paramName.ToUpper
                Case "PORT" : Int32.TryParse(ParamValue, ComPort)
                Case "BAUD" : Int32.TryParse(ParamValue, BaudRate)
                Case "PARITY" : Int32.TryParse(ParamValue, Parity)
                Case "DATABITS" : Int32.TryParse(ParamValue, DataBits)
                Case "STOPBITS" : Int32.TryParse(ParamValue, StopBits)
                Case "DTRON" : Int32.TryParse(ParamValue, DTROn)
                Case "RTSON" : Int32.TryParse(ParamValue, RTSOn)
            End Select

        Next param

        If ComPort <> -1 Then

            Dim IsRunning As Boolean = False

            'Check if the display is already running
            For Each crystalDriver As CrystalFontz634 In CommonFunctions.ConnectedDisplays
                If crystalDriver.Port = ComPort Then
                    IsRunning = True
                    Exit For
                End If
            Next crystalDriver

            'There is no driver running at the moment, great!
            If Not (IsRunning) Then

                'Start a new instance...
                Dim crystalDriver As New CrystalFontz634(BaudRate, Parity, DataBits, StopBits, DTROn, RTSOn)
                crystalDriver.Port = ComPort

                'Try and connect to the port
                If crystalDriver.Connect Then

                    'Report back
                    Call CommonFunctions.SendClientText(sourceClient, "OK: Successfully connected to display on COM" & ComPort & ":")

                    'Store the connected display
                    CommonFunctions.ConnectedDisplays.Add(crystalDriver)

                    'Give any socket that just started the connection a chance to catchup
                    System.Threading.Thread.Sleep(1000)

                Else

                    'Report back
                    Call CommonFunctions.SendClientText(sourceClient, "FAIL: Failed to connect to display on COM" & ComPort & ":")

                End If
            Else
                'Report back
                Call CommonFunctions.SendClientText(sourceClient, "OK: Successfully connected to display already running on COM" & ComPort & ":")
            End If

            'Re-assign the display port this client is using
            sourceClient.DisplayPort = ComPort

        End If

    End Sub

    'Clear a display
    Private Sub ClearDisplay(ByVal SourceClient As DriverClient)

        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then
            sourceDisplay.ClearScreen()
        End If

    End Sub

    'Clear a specific line
    Private Sub ClearLine(ByVal SourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 Then
                    If (sourceDisplay.ClearLine(StartLine)) Then
                        Call CommonFunctions.SendClientText(SourceClient, "OK: Cleared line " & StartLine & ".")
                    Else
                        Call CommonFunctions.SendClientText(SourceClient, "FAILED: Insufficient parameters sent to CLEARLINE command.")
                    End If
                Else
                    Call CommonFunctions.SendClientText(SourceClient, "FAILED: Insufficient parameters sent to CLEARLINE command.")
                End If
            Else
                Call CommonFunctions.SendClientText(SourceClient, "FAILED: Insufficient parameters sent to CLEARLINE command.")
            End If
        Else
            Call CommonFunctions.SendClientText(SourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    'Updates the style content, which can either be the style format, 
    'or the values associated with a style element
    Public Sub UpdateStyleContent(ByVal SourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1
            Dim Style As String = ""
            Dim FlashDelay As Integer = -1
            Dim UpdateIndex As Integer = -1
            Dim StyleType As MultiStyleLine.StyleType = MultiStyleLine.StyleType.Style_None
            Dim UpdateValue As String = ""

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Integer.TryParse(ParamValue, StartLine)
                                Case "STYLE" : Style = ParamValue
                                Case "SPEED" : Integer.TryParse(ParamValue, FlashDelay)
                                Case "STATIC"
                                    Integer.TryParse(ParamValue, UpdateIndex)
                                    StyleType = MultiStyleLine.StyleType.Style_Static
                                Case "SCROLL"
                                    Integer.TryParse(ParamValue, UpdateIndex)
                                    StyleType = MultiStyleLine.StyleType.Style_Scrolling
                                Case "FLASH"
                                    Integer.TryParse(ParamValue, UpdateIndex)
                                    StyleType = MultiStyleLine.StyleType.Style_Flashing
                                Case "VALUE" : UpdateValue = ParamValue
                            End Select

                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If Style > "" And StartLine > -1 Then

                    'Get the appropriate style instance for the specified line
                    Dim sourceStyler As MultiStyleLine = sourceDisplay.MultiStyles(StartLine)
                    sourceStyler.DisplayLine = StartLine

                    'Bind the CrystalFontz display to the styler
                    sourceStyler.SourceDisplay = sourceDisplay

                    'Update the content style 
                    sourceStyler.SourceStyle = Style

                ElseIf Style = "" And StartLine > -1 Then

                    'Get the appropriate style instance for the specified line
                    Dim sourceStyler As MultiStyleLine = sourceDisplay.MultiStyles(StartLine)

                    'Update the content values with the values specified
                    sourceStyler.UpdateLineValue(StyleType, UpdateIndex, UpdateValue)

                End If
            Else
                Call CommonFunctions.SendClientText(SourceClient, "FAILED: Insufficient parameters sent to MULTISTYLE command.")
            End If
        Else
            Call CommonFunctions.SendClientText(SourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    'Modifies the current scroll indexes of a specified line
    Public Sub ModifyScrollIndex(ByVal SourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Integer.TryParse(ParamValue, StartLine)
                            End Select

                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If
                Next param

                If Not (StartLine = -1) Then
                    'Get the appropriate style instance for the specified line
                    Dim sourceStyler As MultiStyleLine = sourceDisplay.MultiStyles(StartLine)
                    sourceStyler.UpdateScrollIndex()
                End If
            End If
        End If

    End Sub

    'Modify the current flash state for the specified line
    Public Sub ModifyFlashIndicator(ByVal SourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(SourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Integer.TryParse(ParamValue, StartLine)
                            End Select

                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If
                Next param

                If Not (StartLine = -1) Then
                    'Get the appropriate style instance for the specified line
                    Dim sourceStyler As MultiStyleLine = sourceDisplay.MultiStyles(StartLine)

                    'Go ahead and modify the flash state!
                    sourceStyler.SetFlash()
                End If
            End If
        End If

    End Sub

    'This code may be of no use to anyone whatsoever.  These are some of the original functions I wrote
    'to control the display, using the classes indicated within the code.

    'If you are not trying to do something complicated, you can always bind your own commands in 'ParseCommands'
    'to re-enable these functions as I once had them.

#Region "Obsolete Code"

    <Obsolete()> _
    Private Sub DisplayText(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1
            Dim StartCol As Integer = -1
            Dim DisplayText As String = ""

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)
            Dim textStart As Integer = paramString.ToLower.IndexOf("/text") + 5

            If paramString > "" Then
                DisplayText = paramString.Substring(textStart, (paramString.Length - textStart)).Trim

                'Add key text support
                DisplayText = CommonFunctions.GetKeyText(DisplayText)

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1).Trim()

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                                Case "COL" : Int32.TryParse(ParamValue, StartCol)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 And StartCol <> -1 And DisplayText <> "" Then
                    If sourceDisplay.SendText(StartLine, StartCol, DisplayText) Then
                        Call CommonFunctions.SendClientText(sourceClient, "OK: Text '" & DisplayText & "' displayed on line " & StartLine & " at column " & StartCol)
                    Else
                        Call CommonFunctions.SendClientText(sourceClient, "FAILED: Text '" & DisplayText & "' could not be displayed on line " & StartLine & " at column " & StartCol)
                    End If
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Text '" & DisplayText & "' could not be displayed on line " & StartLine & " at column " & StartCol)
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to TEXT command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub ScrollText(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1
            Dim ScrollDelay As Integer = 0
            Dim DisplayText As String = ""

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                                Case "SPEED" : Int32.TryParse(ParamValue, ScrollDelay)
                                Case "TEXT" : DisplayText = CommonFunctions.GetKeyText(ParamValue)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 And ScrollDelay <> -1 And DisplayText <> "" Then

                    'Get the appropriate scroller
                    Dim sourceScroller As TextScroller = sourceDisplay.Scrollers(StartLine)

                    'Set the relevant properties on the scroller
                    sourceScroller.ScrollDelay = ScrollDelay
                    sourceScroller.DestinationLine = StartLine
                    sourceScroller.ScrollText = DisplayText
                    sourceScroller.SourceDisplay = sourceDisplay
                    sourceScroller.Start()

                    Call CommonFunctions.SendClientText(sourceClient, "OK: Scrolling text '" & DisplayText & "' on line " & StartLine & " every " & ScrollDelay & " milliseconds.")
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Text '" & DisplayText & "' could not be scrolled on line " & StartLine & " every " & ScrollDelay & " milliseconds.")
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to TEXT command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub StopScroll(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 Then

                    'Get the appropriate scroller
                    Dim sourceScroller As TextScroller = sourceDisplay.Scrollers(StartLine)

                    'Stop the scroller
                    sourceScroller.StopScroll()

                    Call CommonFunctions.SendClientText(sourceClient, "OK: Stopped scrolling text on line " & StartLine & ".")
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPSCROLL command.")
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPSCROLL command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub FlashText(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1
            Dim StartCol As Integer = -1
            Dim FlashDelay As Integer = 0
            Dim DisplayText As String = ""

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then
                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                                Case "COL" : Int32.TryParse(ParamValue, StartCol)
                                Case "SPEED" : Int32.TryParse(ParamValue, FlashDelay)
                                Case "TEXT" : DisplayText = CommonFunctions.GetKeyText(ParamValue)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 And StartCol <> -1 And FlashDelay > 0 And DisplayText <> "" Then

                    Dim sourceFlasher As Flasher = sourceDisplay.Flashers(StartLine)
                    sourceFlasher.DestinationLine = StartLine
                    sourceFlasher.DestinationColumn = StartCol
                    sourceFlasher.FlashingText = DisplayText
                    sourceFlasher.FlashDelay = FlashDelay
                    sourceFlasher.SourceDisplay = sourceDisplay
                    sourceFlasher.Start()
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Text '" & DisplayText & "' could not be displayed on line " & StartLine & " at column " & StartCol)
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to TEXT command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub StopFlash(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 Then

                    'Get the appropriate scroller
                    Dim sourceFlasher As Flasher = sourceDisplay.Flashers(StartLine)

                    'Stop the scroller
                    sourceFlasher.StopFlashing()

                    Call CommonFunctions.SendClientText(sourceClient, "OK: Stopped flashing text on line " & StartLine & ".")
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPFLASH command.")
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPFLASH command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub FlashScrollText(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1
            Dim StartCol As Integer = -1
            Dim FlashDelay As Integer = 0
            Dim FlashText As String = ""
            Dim ScrollText As String = ""

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)
            Dim textStart As Integer = paramString.ToLower.IndexOf("/text") + 5

            If paramString > "" Then
                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                                Case "COL" : Int32.TryParse(ParamValue, StartCol)
                                Case "SPEED" : Int32.TryParse(ParamValue, FlashDelay)
                                Case "FLASH" : FlashText = ParamValue
                                Case "SCROLL" : ScrollText = ParamValue
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 And StartCol <> -1 And FlashDelay > 0 And (FlashText <> "" Or ScrollText <> "") Then

                    Dim sourceFlasher As FlashScroller = sourceDisplay.FlashScrollers(StartLine)
                    sourceFlasher.DestinationLine = StartLine
                    sourceFlasher.DestinationColumn = StartCol
                    sourceFlasher.FlashingText = FlashText
                    sourceFlasher.ScrollText = ScrollText
                    sourceFlasher.FlashDelay = FlashDelay
                    sourceFlasher.SourceDisplay = sourceDisplay
                    sourceFlasher.Start()
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: FLASHSCROLLTEXT could not be displayed on line " & StartLine & " at column " & StartCol)
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to FLASHSCROLLTEXT command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

    <Obsolete()> _
    Private Sub StopFlashScrollText(ByVal sourceClient As DriverClient, ByVal DisplayParams() As String)

        'First, for clarity - strip out for the first parameter
        Dim newParams(DisplayParams.Length - 1) As String
        Array.ConstrainedCopy(DisplayParams, 1, newParams, 0, DisplayParams.Length - 1)

        'Get the current display for the source client
        Dim sourceDisplay As CrystalFontz634 = CommonFunctions.GetCrystalDisplayFromPort(sourceClient)

        If Not (sourceDisplay Is Nothing) Then

            Dim StartLine As Integer = -1

            'Join the string together, and determine the full text
            Dim paramString As String = String.Join("/", DisplayParams, 1, DisplayParams.Length - 1)

            If paramString > "" Then

                'Iterate the other parameters
                For Each param As String In newParams

                    If Not (param Is Nothing) Then
                        Try
                            Dim paramValues() As String = param.Split(Chr(32))

                            Dim paramName As String = paramValues(0)
                            Dim ParamValue As String = String.Join(Chr(32), paramValues, 1, paramValues.Length - 1)

                            Select Case paramName.ToUpper
                                Case "LINE" : Int32.TryParse(ParamValue, StartLine)
                            End Select
                        Catch ex As Exception
                            'Ignore errors, and attempt to carry on
                        End Try
                    End If

                Next param

                If StartLine <> -1 Then

                    'Get the appropriate scroller
                    Dim sourceFlasher As FlashScroller = sourceDisplay.FlashScrollers(StartLine)

                    'Stop the scroller
                    sourceFlasher.StopFlashing()

                    Call CommonFunctions.SendClientText(sourceClient, "OK: Stopped flashing text on line " & StartLine & ".")
                Else
                    Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPFLASH command.")
                End If
            Else
                Call CommonFunctions.SendClientText(sourceClient, "FAILED: Insufficient parameters sent to STOPFLASH command.")
            End If
        Else
            Call CommonFunctions.SendClientText(sourceClient, "FAILED: You have not yet connected to a display.  Use /START to specify a display port")
        End If

    End Sub

#End Region

End Class 'AsynchronousSocketListener
