'=================================
'= CrystalFontz 634 Related Code =
'=================================
'
'A class for managing a CrystalFontz 634 display
'containing a number of functions that are generally
'redundant in the project, but useful for showing how
'to packet general instructions to the display.

'Contains some references to the CRC methods, so again
'potentially this same source could be used as a base for packeting
'data to displays that require the CRC elements to be specified.
'
'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Imports System.IO.Ports

Public Class CrystalFontz634

    Private _Port As Integer
    Private _BaudRate As Integer
    Private _Parity As Parity
    Private _DataBits As Integer
    Private _stopBits As StopBits
    Private _DTREnabled As Boolean = False
    Private _RTSEnabled As Boolean = False

    Private _comPort As New System.IO.Ports.SerialPort

    'A collection of scrollers to house a scrollable line of text for each line of the display
    Friend MultiStyles As New List(Of MultiStyleLine)

#Region "Obselete Variables"
    Friend Scrollers As New List(Of TextScroller)
    Friend Flashers As New List(Of Flasher)
    Friend FlashScrollers As New List(Of FlashScroller)
#End Region

    Public Enum ControlCommands
        CursorHome = 1
        HideDisplay = 2
        RestoreDisplay = 3
        HideCursor = 4
        ShowUnderlineCursor = 5
        ShowBlockCursor = 6
        ShowInvertingBlockCursor = 7
        Backspace = 8
        BootScreenOperation = 9
        LineFeed = 10
        DeleteInPlace = 11
        FormFeed = 12
        CarriageReturn = 13
        BackLightControl = 14
        ContrastControl = 15
        SetCursorPosition = 17
        HorizontalBar = 18
        ScrollOn = 19
        ScrollOff = 20
        SetScrollingMarqueeCharacter = 21
        EnableScrollingMarquee = 22
        WrapOn = 23
        WrapOff = 24
        SetCustomBootCharacter = 25
        Reboot = 26
        EscapePrefix = 27
        LargeBlockNumber = 28
        DirectData = 30
        ShowLCDInfo = 31
    End Enum

    'Any specific settings required by the LCD that need to be used to make the connection
    Public Property Port() As Integer
        Get
            Return _Port
        End Get
        Set(ByVal value As Integer)
            _Port = value
        End Set
    End Property

    Public Sub New(ByVal BaudRate As Integer, ByVal Parity As Parity, ByVal DataBits As Integer, ByVal StopBits As StopBits, ByVal DTREnabled As Boolean, ByVal RTSEnabled As Boolean)

        _BaudRate = BaudRate
        _Parity = Parity
        _DataBits = DataBits
        _stopBits = StopBits
        _DTREnabled = DTREnabled
        _RTSEnabled = RTSEnabled

        Call Reset()

    End Sub

    Public Function Connect() As Boolean

        Dim success As Boolean = False

        _comPort = New SerialPort("COM" & _Port, _BaudRate, _Parity, _DataBits, _stopBits)
        _comPort.DtrEnable = _DTREnabled
        _comPort.RtsEnable = _RTSEnabled

        Try
            _comPort.Open()

            'Clear the display ... although this generally will be done on connect
            Call HideCursor()
            Call ClearScreen()
            Call SetWrapEnabled(False)

            success = True
        Catch ex As Exception

        End Try

        Return success

    End Function

    Public Sub SendCursorHome()

        Dim nextPacket As New CrystalFontzPacket

        nextPacket.DataCommand = ControlCommands.CursorHome
        Call SendPacket(nextPacket)

    End Sub

    Public Sub SetWrapEnabled(ByVal Enabled As Boolean)

        Dim nextPacket As New CrystalFontzPacket
        nextPacket.DataCommand = IIf(Enabled, ControlCommands.WrapOn, ControlCommands.WrapOff)
        Call SendPacket(nextPacket)

    End Sub

    Public Sub ClearScreen()

        Call SendCursorHome()

        For lineCount As Integer = 0 To 3
            Call SendText(lineCount, 0, Space(20))
        Next lineCount

        Call SendCursorHome()

    End Sub

    Public Function ClearLine(ByVal LineIndex As Integer) As Boolean

        Dim DisplayText As String = Space(20)

        Return SendText(LineIndex, 0, DisplayText)

    End Function

    Public Sub HideCursor()

        Dim nextPacket As New CrystalFontzPacket
        nextPacket.DataCommand = ControlCommands.HideCursor

        Call SendPacket(nextPacket)

    End Sub

    Public Function DirectWriteSpecialChar(ByVal Character As String) As Boolean

        Dim nextPacket As New CrystalFontzPacket
        Dim ReturnValue As Boolean = True

        'Build up the special character command
        nextPacket.DataCommand = ControlCommands.DirectData

        'Set the parameters for the packet command
        nextPacket.DataCommandParams = New Byte() {1}

        'Now add the character
        nextPacket.Data = System.Text.ASCIIEncoding.ASCII.GetBytes(Character)

        Try
            'Send it down the line and pray!
            Call SendPacket(nextPacket)

        Catch ex As Exception

            ReturnValue = False

        End Try

        Return ReturnValue

    End Function

    Public Function GetSpecialCharBytes(ByVal Character As String) As Byte()

        Dim nextPacket As New CrystalFontzPacket
        Dim returnBytes() As Byte

        'Build up the special character command
        nextPacket.DataCommand = ControlCommands.DirectData

        'Set the parameters for the packet command
        nextPacket.DataCommandParams = New Byte() {1}

        'Now add the character
        nextPacket.Data = System.Text.ASCIIEncoding.ASCII.GetBytes(Character)

        Try
            'Send it down the line and pray!
            returnBytes = nextPacket.GetBytes()

        Catch ex As Exception

            returnBytes = Nothing

        End Try

        Return returnBytes

    End Function

    Public Function SendSpecialChar(ByVal Character As String, ByVal Line As Integer, ByVal Column As Integer)

        Dim nextPacket As New CrystalFontzPacket
        Dim ReturnValue As Boolean = True

        'Mark the cursor position
        nextPacket.DataCommand = ControlCommands.SetCursorPosition

        'Set the parameters for the packet command
        nextPacket.DataCommandParams = New Byte() {Column, Line}

        'Now send the packet down the port
        Try
            Call SendPacket(nextPacket)

            'Now send the special character
            nextPacket.DataCommand = ControlCommands.DirectData

            'Set the parameters for the packet command
            nextPacket.DataCommandParams = New Byte() {1}

            'Now add the character
            nextPacket.Data = System.Text.ASCIIEncoding.ASCII.GetBytes(Character)

            'Send it down the line and pray!
            Call SendPacket(nextPacket)

        Catch ex As Exception

            ReturnValue = False

        End Try

        Return ReturnValue

    End Function

    Public Function GetCursorPositionBytes(ByVal Line As Integer, ByVal Column As Integer) As Byte()

        'Set the parameters for the packet command
        Return New Byte() {ControlCommands.SetCursorPosition, Column, Line}

    End Function

    Public Function SetCursorPosition(ByVal Line As Integer, ByVal Column As Integer)

        Dim nextPacket As New CrystalFontzPacket
        Dim ReturnValue As Boolean = True

        'Mark the cursor position
        nextPacket.DataCommand = ControlCommands.SetCursorPosition

        'Set the parameters for the packet command
        nextPacket.DataCommandParams = New Byte() {Column, Line}

        'Now send the packet down the port
        Try
            Call SendPacket(nextPacket)

        Catch ex As Exception

            ReturnValue = False

        End Try

        Return ReturnValue

    End Function

    Public Function SendText(ByVal Line As Integer, ByVal Column As Integer, ByVal Text As String) As Boolean

        Dim nextPacket As New CrystalFontzPacket
        Dim ReturnValue As Boolean = True

        'Mark the cursor position
        nextPacket.DataCommand = ControlCommands.SetCursorPosition

        'Set the parameters for the packet command
        nextPacket.DataCommandParams = New Byte() {Column, Line}

        'Set the data part of the command
        nextPacket.Data = System.Text.Encoding.ASCII.GetBytes(Text)

        'Now send the packet down the port
        Try
            Call SendPacket(nextPacket)

        Catch ex As Exception

            ReturnValue = False

        End Try

        Return ReturnValue

    End Function

    Private Sub SendPacket(ByVal PacketToSend As CrystalFontzPacket)

        'A message, at it's minimum can be 1 byte, excluding the command + CRC
        Dim MessageLen As Integer = 1

        'Now determine how long the message data really is - first any parameters
        'specified for the command?
        If Not (PacketToSend.DataCommandParams Is Nothing) Then
            MessageLen += PacketToSend.DataCommandParams.Length
        End If

        'What about data, any of that?
        If Not (PacketToSend.Data Is Nothing) Then
            MessageLen += PacketToSend.Data.Length
        End If

        'Now add two, for CRC based controlled LCDs
        'MessageLen += 1

        'Now build a single array to construct this packet message
        Dim PacketMessage(MessageLen - 1) As Byte
        Dim copyIndex As Integer = 1

        'Copy in the data command
        PacketMessage(0) = PacketToSend.DataCommand

        'Now, the command parameters
        If Not (PacketToSend.DataCommandParams Is Nothing) Then
            For nextIndex As Integer = 0 To PacketToSend.DataCommandParams.Length - 1
                PacketMessage(copyIndex) = PacketToSend.DataCommandParams(nextIndex)
                copyIndex += 1
            Next nextIndex
        End If

        'Now the data
        If Not (PacketToSend.Data Is Nothing) Then
            For nextIndex As Integer = 0 To PacketToSend.Data.Length - 1
                PacketMessage(copyIndex) = PacketToSend.Data(nextIndex)
                copyIndex += 1
            Next nextIndex
        End If

        'Write the message to the display
        _comPort.Write(PacketMessage, 0, PacketMessage.Length)

    End Sub

    Public Function DirectWrite(ByVal Bytes() As Byte) As Boolean

        Try
            _comPort.Write(Bytes, 0, Bytes.Length)
        Catch ex As Exception

        End Try
    End Function

    Public Sub Close()

        _comPort.Close()

    End Sub

    Public Function GetCGROMBytes(ByVal sourceCharacter As Integer) As Byte()

        Dim commandBytes(2) As Byte

        commandBytes(0) = 30
        commandBytes(1) = 1
        commandBytes(2) = sourceCharacter

        Return commandBytes

    End Function

    Public Sub Reset()

        Call DestroyInstances(Scrollers)
        Call DestroyInstances(Flashers)
        Call DestroyInstances(FlashScrollers)
        Call DestroyInstances(MultiStyles)

        'Initialise the scroll timers
        For scrollerCount As Integer = 0 To 3
            Scrollers.Add(New TextScroller)
            Flashers.Add(New Flasher)
            FlashScrollers.Add(New FlashScroller)
            MultiStyles.Add(New MultiStyleLine)
        Next scrollerCount

    End Sub

    Private Sub DestroyInstances(ByRef sourceObject As Object)

        For itemCount As Integer = sourceObject.Count - 1 To 0 Step -1

            'Get a reference to the object
            Dim sourceRef As Object = sourceObject.Item(itemCount)

            'Remove it from the list
            sourceObject.Remove(sourceRef)

            'Clean it up
            sourceRef = Nothing
        Next itemCount

    End Sub

End Class
