'=======================================
'= LCD Multiple Style Line Source Code =
'=======================================

'The only thing really needed to display anything on the LCD.
'Allows a style format to be embedded and the associated data
'displayed on the screen according to the format.

'Supports multiple static, flashing and scrolling elements within a single line
'although some element styles cannot be combined (ie flashing, scrolling message)

'Written by Stuart Massey (VorTechS)
'3rd February 2010

'LICENSE INFORMATION:
'
'This file is part of the PhatHack LCD Driver software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.

'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.

'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Public Class MultiStyleLine

    Private _SourceString As String = ""
    Private _StyleString As String = ""
    Private IsFlashOn As Boolean = False
    Private _ScrollLengths As New List(Of Integer)
    Private _ScrollIndexes As New List(Of Integer)
    Private _ScrollStrings As New List(Of String)
    Private _StaticStrings As New List(Of String)
    Private _FlashingStrings As New List(Of String)

    Private _DestinationLine As Integer

    Private _OutputBytes() As Byte

    Private synchObject As New Object

    Public Enum StyleType
        Style_None = -1
        Style_Static = 0
        Style_Flashing = 1
        Style_Scrolling = 2
    End Enum

    Private _sourceDisplay As CrystalFontz634 = Nothing

    Public Sub New()

    End Sub

    Private Sub ParseSourceString()

        Dim IsCommandOpen As Boolean = False
        Dim IsSpecialCharOpen As Boolean = False
        Dim IsSettingScrollLen As Boolean = False

        Dim CurrentCommand As StyleType = StyleType.Style_None
        Dim CurrentScrollLen As String = ""

        Dim ScrollCounts As Integer = -1
        Dim StaticCounts As Integer = -1
        Dim FlashCounts As Integer = -1

        'Clearing the relevant buffers to keep things clean
        _ScrollLengths.Clear()
        _ScrollStrings.Clear()
        _ScrollIndexes.Clear()

        _StaticStrings.Clear()
        _FlashingStrings.Clear()

        'Now iterate through the source string, build up the relevant
        'parameters and overall style string for the display
        For Each character As String In _SourceString

            Dim SendIt As Boolean = True

            If character = "[" And Not (IsCommandOpen) Then

                If CurrentCommand = StyleType.Style_Scrolling Then
                    _ScrollStrings(ScrollCounts) &= Space(_ScrollLengths(ScrollCounts))
                End If
                CurrentCommand = StyleType.Style_None
                IsCommandOpen = True
                SendIt = False
            ElseIf character = "]" And IsCommandOpen Then
                IsCommandOpen = False
                SendIt = False

                If IsSettingScrollLen Then
                    ScrollCounts += 1

                    Dim scrollLen As Integer = -1

                    Call Integer.TryParse(CurrentScrollLen, scrollLen)

                    _ScrollLengths.Add(scrollLen)

                    If _ScrollIndexes.Count < (ScrollCounts + 1) Then
                        _ScrollIndexes.Add(0)
                    End If

                    _ScrollLengths(ScrollCounts) = CurrentScrollLen
                    _ScrollStrings.Add(Space(CurrentScrollLen))

                    CurrentScrollLen = ""
                    IsSettingScrollLen = False
                ElseIf CurrentCommand = StyleType.Style_Static Then
                    StaticCounts += 1
                    _StaticStrings.Add("")
                ElseIf CurrentCommand = StyleType.Style_Flashing Then
                    FlashCounts += 1
                    _FlashingStrings.Add("")
                End If
            ElseIf IsCommandOpen Then

                If CurrentCommand = StyleType.Style_None Then
                    Call Integer.TryParse(character, CurrentCommand)
                End If

                If CurrentCommand = StyleType.Style_Scrolling Then

                    If character = "*" And IsCommandOpen Then
                        IsSettingScrollLen = True
                    ElseIf IsSettingScrollLen Then
                        CurrentScrollLen &= character
                    End If
                End If
                SendIt = False
            End If


            'Build the style string, used when rendering the display
            If Not (SendIt) And Not (IsSpecialCharOpen) Then
                _StyleString &= character
            End If

            'Evaluate the current command
            If CurrentCommand = StyleType.Style_Scrolling And Not (IsCommandOpen) And SendIt Then
                'Append to the source string
                _ScrollStrings(ScrollCounts) &= character
            ElseIf CurrentCommand = StyleType.Style_Static And Not (IsCommandOpen) And SendIt Then
                'Append to the source string
                _StaticStrings(StaticCounts) &= character
            ElseIf CurrentCommand = StyleType.Style_Flashing And Not (IsCommandOpen) And SendIt Then
                'Append to the source string
                _FlashingStrings(FlashCounts) &= character
            End If
        Next character

        'Complete the strings where we get to the end of the strings with our current command
        If CurrentCommand = StyleType.Style_Scrolling Then
            _ScrollStrings(ScrollCounts) &= Space(_ScrollLengths(ScrollCounts))
        End If

    End Sub

    Public Sub DisplayStyle()

        SyncLock synchObject

            If Not (_sourceDisplay Is Nothing) Then
                'Reset the byte array
                _OutputBytes = _sourceDisplay.GetCursorPositionBytes(_DestinationLine, 0)

                Dim IsCommandOpen As Boolean = False
                Dim IsSpecialCharOpen As Boolean = False
                Dim IsSettingScrollLen As Boolean = False

                Dim CurrentCommand As StyleType = StyleType.Style_None
                Dim CurrentScrollLen As String = ""
                Dim CurrentSpecialChar As String = ""

                Dim ScrollCounts As Integer = 0
                Dim StaticCounts As Integer = 0
                Dim FlashCounts As Integer = 0

                'Now iterate through the buffer, and perform the relevant functions
                For Each character As String In _StyleString

                    Dim SendIt As Boolean = True

                    If character = "[" And Not (IsCommandOpen) Then
                        CurrentCommand = StyleType.Style_None
                        IsCommandOpen = True
                        SendIt = False
                    ElseIf character = "]" And IsCommandOpen Then
                        IsCommandOpen = False
                        SendIt = True
                    ElseIf IsCommandOpen Then
                        If CurrentCommand = StyleType.Style_None Then
                            Call Integer.TryParse(character, CurrentCommand)
                        End If
                        SendIt = False
                    End If

                    'Evaluate the current command
                    If CurrentCommand = StyleType.Style_Scrolling And Not (IsCommandOpen) And SendIt Then

                        'Find out what the current scrolling index is
                        Dim PartialString As String = ""
                        Dim TempString As String = CommonFunctions.GetKeyText(_ScrollStrings(ScrollCounts))

                        If _ScrollIndexes(ScrollCounts) + _ScrollLengths(ScrollCounts) < _ScrollStrings(ScrollCounts).Length Then
                            PartialString = TempString.Substring(_ScrollIndexes(ScrollCounts), _ScrollLengths(ScrollCounts))
                        Else
                            PartialString = TempString.Substring(_ScrollIndexes(ScrollCounts))
                        End If

                        Call DirectWriteSourceString(CurrentCommand, PartialString, IsFlashOn)

                        ScrollCounts += 1
                    ElseIf CurrentCommand = StyleType.Style_Static And Not (IsCommandOpen) And SendIt Then
                        Dim TempString As String = CommonFunctions.GetKeyText(_StaticStrings(StaticCounts))
                        Call DirectWriteSourceString(CurrentCommand, TempString, False)
                        StaticCounts += 1
                    ElseIf CurrentCommand = StyleType.Style_Flashing And Not (IsCommandOpen) And SendIt Then
                        Dim TempString As String = CommonFunctions.GetKeyText(_FlashingStrings(FlashCounts))
                        Call DirectWriteSourceString(CurrentCommand, TempString, IsFlashOn)
                        FlashCounts += 1
                    End If
                Next character
            End If

        End SyncLock

    End Sub

    Private Sub CopyArrayBytes(ByVal SourceArray() As Byte, ByRef DestinationArray() As Byte)

        'Assumes the source array is zero bound, and one more element to it
        Dim startIndex As Integer = 0

        If (DestinationArray Is Nothing) Then
            ReDim DestinationArray(SourceArray.Length - 1)
        Else
            startIndex = DestinationArray.Length
            ReDim Preserve DestinationArray((SourceArray.Length - 1) + DestinationArray.Length)
        End If

        SourceArray.CopyTo(DestinationArray, startIndex)

    End Sub

    Private Sub DirectWriteSourceString(ByVal CurrentCommand As StyleType, ByVal SourceString As String, ByVal FlashOn As Boolean)

        Dim IsSpecialCharOpen As Boolean = False
        Dim specialCharPhrase As String = ""
        Dim outputBytes() As Byte = Nothing

        For Each textChar As String In SourceString
            Dim SendIt As Boolean = True
            Dim GetFinalChar As Boolean = False

            If (textChar = "{") And Not (IsSpecialCharOpen) Then
                SendIt = False
                IsSpecialCharOpen = True
            End If

            If (textChar = "}") And (IsSpecialCharOpen) Then
                SendIt = False
                GetFinalChar = True
                IsSpecialCharOpen = False
            End If


            If Not (IsSpecialCharOpen) And SendIt Then
                Dim sourcebytes() As Byte
                If FlashOn Or (CurrentCommand <> StyleType.Style_Flashing) Then
                    'Send the special character to the display
                    sourcebytes = System.Text.ASCIIEncoding.ASCII.GetBytes(textChar)
                    Call CopyArrayBytes(sourcebytes, outputBytes)
                Else
                    'Write this character directly to the display
                    sourcebytes = System.Text.ASCIIEncoding.ASCII.GetBytes(" ")
                    Call CopyArrayBytes(sourcebytes, outputBytes)
                End If
            ElseIf (IsSpecialCharOpen) And SendIt Then
                'Build up the numerics to get the special char from
                specialCharPhrase &= textChar
            ElseIf Not (IsSpecialCharOpen) And Not (SendIt) And GetFinalChar Then
                Dim sourcebytes() As Byte

                If FlashOn Or (CurrentCommand <> StyleType.Style_Flashing) Then

                    Dim SpecialCharNum As Integer = 0

                    Call Integer.TryParse(specialCharPhrase, SpecialCharNum)

                    Select Case SpecialCharNum
                        Case 16 To 31, 128 To 135
                            'Write this character directly to the display
                            sourcebytes = _sourceDisplay.GetCGROMBytes(SpecialCharNum)
                            Call CopyArrayBytes(sourcebytes, outputBytes)
                        Case Else
                            'Write this character directly to the display
                            sourcebytes = System.Text.ASCIIEncoding.ASCII.GetBytes(Chr(specialCharPhrase))
                            Call CopyArrayBytes(sourcebytes, outputBytes)
                    End Select
                Else
                    'Write this character directly to the display
                    sourcebytes = System.Text.ASCIIEncoding.ASCII.GetBytes(" ")
                    Call CopyArrayBytes(sourcebytes, outputBytes)
                End If

                'Reset the phrase back to blank
                specialCharPhrase = ""
            End If
        Next textChar

        'Reset the byte array
        Call CopyArrayBytes(outputBytes, _OutputBytes)

    End Sub

    Public Function UpdateLineValue(ByVal StringType As StyleType, ByVal Index As Integer, ByVal NewValue As String) As Boolean

        Dim returnValue = False

        Try
            Select Case StringType
                Case StyleType.Style_Flashing
                    If _FlashingStrings.Count > Index Then
                        _FlashingStrings(Index) = NewValue
                        returnValue = True
                    End If
                Case StyleType.Style_Scrolling
                    If _ScrollStrings.Count > Index Then
                        _ScrollStrings(Index) = Space(_ScrollLengths(Index)) & NewValue & Space(_ScrollLengths(Index))
                        returnValue = True
                    End If
                Case StyleType.Style_Static
                    If _StaticStrings.Count > Index Then
                        _StaticStrings(Index) = NewValue
                        returnValue = True
                    End If
            End Select
        Catch
        End Try

        Return returnValue
    End Function

    Public Property SourceDisplay() As CrystalFontz634
        Get
            Return _sourceDisplay
        End Get
        Set(ByVal value As CrystalFontz634)
            _sourceDisplay = value
        End Set
    End Property

    Public Property SourceStyle() As String
        Get
            Return _SourceString
        End Get
        Set(ByVal value As String)

            _SourceString = value

            'Configure the mutli-style line based on what's been provided
            Call ParseSourceString()

        End Set
    End Property

    Public Property DisplayLine() As Integer
        Get
            Return _DestinationLine
        End Get
        Set(ByVal value As Integer)
            _DestinationLine = value
        End Set
    End Property

    Public ReadOnly Property GetBytes() As Byte()
        Get
            Return _OutputBytes
        End Get
    End Property

    Public Sub SetBytes(ByVal sourceBytes() As Byte)

        _OutputBytes = sourceBytes

    End Sub

    Public Sub UpdateScrollIndex()

        Try

            'Iterate each scroll element in the current display
            For Element As Integer = 0 To _ScrollIndexes.Count - 1

                'Validate the index, and reset it if necessary
                If _ScrollIndexes(Element) + _ScrollLengths(Element) > _ScrollStrings(Element).Length Then
                    _ScrollIndexes(Element) = -1
                End If

                'Increment the scroll index
                _ScrollIndexes(Element) += 1

            Next Element

        Catch ex As Exception
            'It's possible for a timed event to try and update the scroll index state - whilst the style
            'is being applied etc etc - so just ignore any errors at this point
        End Try

    End Sub

    Public Sub SetFlash()

        Try
            'Turn the flash on/off by toggling the current state
            IsFlashOn = Not (IsFlashOn)
        Catch ex As Exception

        End Try

    End Sub
End Class


